#include <zephyr/logging/log.h>
#include <zephyr/zbus/zbus.h>

#include <stdio.h>
#include <string.h>

#include "utilities.h"
#include "ble-scanner.h"

LOG_MODULE_REGISTER(BleScanner);
static BleScanner *context;

static struct bt_le_scan_param scan_param = {
		.type       = BT_LE_SCAN_TYPE_PASSIVE,
		.options    = BT_LE_SCAN_OPT_NONE,
		.interval   = BT_GAP_SCAN_FAST_INTERVAL,
		.window     = BT_GAP_SCAN_FAST_WINDOW,
	};

BleScanner::BleScanner(){
    context = this;
    int err = bt_enable(NULL);
	LOG_INF("start bt");
	if (err != 0) {
		printk("Bluetooth init failed (err %d)\n", err);
		return;
	}

	k_work_init(&bleWork, BleInterruptHandler);
}

void BleScanner::BleInterruptHandler(struct k_work *item){
	if(context->registeredCallback != NULL){
		context->registeredCallback(context->currAddr, context->currData, context->currRssi);
	}
}

void BleScanner::SetScan(bool enable){
	if(enable){
		bt_le_scan_start(&scan_param, BleDevicefound);
	}
	else{
		bt_le_scan_stop();	
	}
}

void BleScanner::RegisterCallback(void (*callback)(std::string, std::vector<uint8_t>, int8_t)){
	registeredCallback = callback;
}

RawBleData_t BleScanner::GetLatestData(){
	RawBleData_t rawData = {
		.addrStr = currAddr,
		.bleData = currData,
		.rssi = currRssi
	};
	return rawData;
}

void BleScanner::BleDevicefound(const bt_addr_le_t *addr, int8_t rssi, uint8_t type, struct net_buf_simple *ad){
    char addr_str[BT_ADDR_LE_STR_LEN];
	bt_addr_le_to_str(addr, addr_str, sizeof(addr_str));
	std::string addrStr(addr_str, sizeof(addr_str));
	std::vector<uint8_t> dataVec(ad->data, ad->data + ad->len);

	context->currAddr = addrStr;
	context->currData = dataVec;
	context->currRssi = rssi;

	int err = k_work_submit(&context->bleWork);
}
