#include "ir-driver.h"
#include <zephyr/drivers/gpio.h>

#define IRSENSOR_PIN DT_NODELABEL(ir_sensor)

struct gpio_dt_spec irSensorNode = GPIO_DT_SPEC_GET(IRSENSOR_PIN, gpios);
struct gpio_callback irSensorCb;

static IrDriver *context;

IrDriver::IrDriver(){
    context = this;
    int err = gpio_pin_configure_dt(&irSensorNode, GPIO_INPUT);
    gpio_pin_interrupt_configure_dt(&irSensorNode, GPIO_INT_EDGE_BOTH);
    gpio_init_callback(&irSensorCb, InterruptCallback, BIT(irSensorNode.pin));
	gpio_add_callback(irSensorNode.port, &irSensorCb);

    k_work_init(&work, InterruptHandler);
}

IrDriver::~IrDriver(){

}

bool IrDriver::GetState(){

    return gpio_pin_get_dt(&irSensorNode);
}

void IrDriver::RegisterInterrupt(void (*callback)(uint8_t)){
    registeredCallback = callback;
}

void IrDriver::InterruptCallback(const struct device *dev, struct gpio_callback *cb, uint32_t pins){
    k_work_submit(&context->work);
}

void IrDriver::InterruptHandler(struct k_work *item){
    if(context->registeredCallback != NULL){
        context->registeredCallback(gpio_pin_get_dt(&irSensorNode));
    }
}