#include "servo-driver.h"
#include <zephyr/drivers/pwm.h>
#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(Servo);

#define MIN_PERIOD PWM_SEC(1U) / 128U
#define MAX_PERIOD PWM_SEC(1U)

#define MAX_PWM 100

static const struct pwm_dt_spec pwm_led0 = PWM_DT_SPEC_GET(DT_ALIAS(pwm_led0));
static const uint32_t minPulse = PWM_USEC(1000);
static const uint32_t maxPulse = PWM_USEC(2000);


ServoDriver::ServoDriver(){
	uint8_t dir = 0U;
	int ret;

    if (!pwm_is_ready_dt(&pwm_led0)) {
		LOG_ERR("Error: PWM device %s is not ready\n",
		       pwm_led0.dev->name);
	}

    /*
	 * In case the default MAX_PERIOD value cannot be set for
	 * some PWM hardware, decrease its value until it can.
	 *
	 * Keep its value at least MIN_PERIOD * 4 to make sure
	 * the sample changes frequency at least once.
	 */
	// LOG_INF("Calibrating for channel %d...\n", pwm_led0.channel);
	// max_period = MAX_PERIOD;
	// while (pwm_set_dt(&pwm_led0, max_period, max_period / 2U) < 0) {
	// 	max_period /= 2U;
	// 	if (max_period < (4U * MIN_PERIOD)) {
	// 		LOG_INF("Error: PWM device "
	// 		       "does not support a period at least %lu\n",
	// 		       4U * MIN_PERIOD);
	// 	}
	// }
}

ServoDriver::~ServoDriver(){

}

void ServoDriver::SetPwm(uint8_t pwm){
	if(period != pwm){
		uint32_t pulse = minPulse + ((maxPulse - minPulse) * (pwm / MAX_PWM));
		if(pwm_set_dt(&pwm_led0, max_period, pulse) == 0){
			period = pwm;
		}
	}
}

uint8_t ServoDriver::GetPwm(){
    return period;
}