#include <zephyr/logging/log.h>
#include <zephyr/zbus/zbus.h>

#include <stdio.h>
#include <string.h>

#include "esp_private/system_internal.h"

#include "utilities.h"
#include "wifi-controller.h"
#include <zephyr/net/net_ip.h>
#include <zephyr/net/net_if.h>
#include <zephyr/net/net_event.h>
#include <zephyr/net/wifi_mgmt.h>
#include <zephyr/net/wifi_utils.h>
#include <zephyr/net/net_mgmt.h>
// #include <zephyr/posix/unistd.h>
#include <zephyr/sys/slist.h>
#include <zephyr/init.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/sys/reboot.h>

#define AUTO_CONNECT_SSID "tselhome-605E"
#define AUTO_CONNECT_SSID_PSK "Sapiular0"

#define WIFI_MGMT_EVENTS (NET_EVENT_WIFI_SCAN_RESULT |		\
				NET_EVENT_WIFI_SCAN_DONE |		\
				NET_EVENT_WIFI_CONNECT_RESULT |		\
				NET_EVENT_WIFI_DISCONNECT_RESULT)

#define WIFI_LED DT_NODELABEL(wifipin)

LOG_MODULE_REGISTER(WifiController);
// ZBUS_CHAN_DECLARE(driverRequestChannel, bleScanChannel);

struct gpio_dt_spec wifiLedNode = GPIO_DT_SPEC_GET(WIFI_LED, gpios);

static WifiController *context;
static struct k_sem net_cb_sem;
static struct net_mgmt_event_callback cb;


WifiController::WifiController(){
    context = this;
    struct wifi_connect_req_params wifi_args;

	wifi_args.security = WIFI_SECURITY_TYPE_PSK;
	wifi_args.channel = WIFI_CHANNEL_ANY;
	wifi_args.psk = (const uint8_t *)AUTO_CONNECT_SSID_PSK;
	wifi_args.psk_length = strlen(AUTO_CONNECT_SSID_PSK);
	wifi_args.ssid = (const uint8_t *)AUTO_CONNECT_SSID;
	wifi_args.ssid_length = strlen(AUTO_CONNECT_SSID);

	gpio_pin_configure_dt(&wifiLedNode, GPIO_OUTPUT_LOW);

	// Init semaphore
	k_sem_init(&net_cb_sem, 0, 1);

	// Configure Callback
	net_mgmt_init_event_callback(&cb, WifiEventCallback, WIFI_MGMT_EVENTS );
	net_mgmt_add_event_callback(&cb);

	// Connect interface to network
	struct net_if *iface = net_if_get_default();
	if(iface == NULL){
		LOG_ERR("iface NULL");
		return;
	}

	if(iface->if_dev == NULL){
		LOG_ERR("iface->if_dev NULL");
		return;
	}
	int ret = net_mgmt(NET_REQUEST_WIFI_CONNECT, iface, &wifi_args, sizeof(wifi_args));
	if(ret != 0) {
		LOG_ERR("Failed to request connection to SSID %s", AUTO_CONNECT_SSID);
	}

	// Wait for connection.....
	LOG_INF("Wait for connection.....");
	if(k_sem_take(&net_cb_sem, K_MSEC(5000)) != 0){
		// sys_reboot(SYS_REBOOT_COLD);
		esp_restart_noos();
	}
	LOG_INF("SUCCESFULLY CONNECTED");
}

void WifiController::WifiEventCallback( struct net_mgmt_event_callback *cb, uint32_t mgmt_event, struct net_if *iface ){
	switch( mgmt_event ) {
		case NET_EVENT_WIFI_CONNECT_RESULT:
			const struct wifi_status *status = (const struct wifi_status *) cb->info;
			if (!status->status) {
				// Connected
				k_sem_give(&net_cb_sem);
				gpio_pin_set_dt(&wifiLedNode, true);
			}
			break;
	}
}