#pragma once

#include <zephyr/kernel.h>
#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/hci.h>
#include <string>
#include <vector>

typedef struct RawBleData {
    std::string addrStr; 
    std::vector<uint8_t> bleData; 
    int8_t rssi;
} RawBleData_t;

class BleScanner {
public:
    /** Constructor.
     *
     **/
    BleScanner();

    /** Destructor.
     *
     **/
    ~BleScanner();

    static void DriverRequestListener(const struct zbus_channel *chan);

    void RegisterCallback(void (*callback)(std::string, std::vector<uint8_t>, int8_t));

    void SetScan(bool enable);

    RawBleData_t GetLatestData();

private:
    std::string currAddr;
    std::vector<uint8_t> currData;
    int8_t currRssi;

    struct k_work bleWork;

    void (*registeredCallback)(std::string, std::vector<uint8_t>, int8_t) = NULL;

    static void BleDevicefound(const bt_addr_le_t *addr, int8_t rssi, uint8_t type, struct net_buf_simple *ad);

    static void BleInterruptHandler(struct k_work *item);
    
        
};