#include "device-parser.h"
#include <zephyr/logging/log.h>
#include <zephyr/zbus/zbus.h>

#ifndef CONFIG_SOC_POSIX
LOG_MODULE_REGISTER(DeviceParser, LOG_LEVEL_INF);
#else
LOG_MODULE_REGISTER(DeviceParser, LOG_LEVEL_DBG);
#endif

ZBUS_CHAN_DECLARE(bleScanChannel, driverResponseChannel);

static DeviceParser *context;

ZBUS_LISTENER_DEFINE(scanListener, context->BleScanListener);

DeviceParser::DeviceParser(){
    context = this;
    zbus_chan_add_obs(&bleScanChannel, &scanListener, K_NO_WAIT);
}

void DeviceParser::BleScanListener(const struct zbus_channel *chan){
	const BleData_t *msg;
    if (&bleScanChannel == chan) {
			msg = (BleData_t *)zbus_chan_const_msg(chan); // Direct message access
            BleSensorData_t parsedData;
			if(context->BleDeviceParser(msg, &parsedData)){
                LOG_DBG("Device %s has data %d", msg->address.c_str(), parsedData.sensorData[0]);
                std::vector<uint8_t> dataValue;
                dataValue.push_back(parsedData.sensorData[0]);
                DriverMessage_t sentMessage = {
                    .number = BLUETOOTH,
                    .type = REQUEST_SET,
                    .value = dataValue
                };

                zbus_chan_pub(&driverResponseChannel, &sentMessage, K_FOREVER);
            }
            LOG_HEXDUMP_DBG(msg->data.data(), msg->data.size(), "data raw");
	}
    else{
        return;
    }
}

bool DeviceParser::BleDeviceParser(const BleData_t *data, BleSensorData_t *parsedData){
    if(data->data.size() >= 8){
        if(data->data[5] == 0x1a && data->data[6] == 0x18){
            parsedData->length = (data->data[7] << 8) | (data->data[8]);
            for(int i = 0; i < parsedData->length; i++){
                parsedData->sensorData[i] = data->data[9 + i];
            }
            return true;
        }
    }
    return false;
}