#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <zephyr/zbus/zbus.h>

#include <stdio.h>
#include <string.h>

#include "main-handler.h"

#define MAIN_THREAD_STACK_SIZE 2048
#define MAIN_THREAD_PRIORITY 5

#ifndef CONFIG_SOC_POSIX
LOG_MODULE_REGISTER(Main_Handler, LOG_LEVEL_INF);
#else
LOG_MODULE_REGISTER(Main_Handler, LOG_LEVEL_DBG);
#endif

// Internal
static MainHandler *context;

K_THREAD_STACK_DEFINE(mainThreadStack, MAIN_THREAD_STACK_SIZE);
ZBUS_CHAN_DECLARE(driverResponseChannel, driverRequestChannel, stateChannel);

ZBUS_SUBSCRIBER_DEFINE(stateSubs, 1);
// ZBUS_SUBSCRIBER_DEFINE(sensorSubs, 1);
ZBUS_LISTENER_DEFINE(driverListener, context->DriverResponseListener);

// ZBUS_CHAN_ADD_OBS(driverResponseChannel, sensorSubs, 3);


// Thread and kernels
struct k_thread mainThreadHandler;
k_tid_t mainThreadId;
static struct k_sem responseWait;

MainHandler::MainHandler(){
	context = this;
    mainThreadId = k_thread_create(&mainThreadHandler, mainThreadStack, MAIN_THREAD_STACK_SIZE, 
									  (k_thread_entry_t) MainThread, 
									  this, NULL, NULL, 
									  MAIN_THREAD_PRIORITY, 0, K_NO_WAIT);

	k_sem_init(&responseWait, 0, 1);

	// zbus_chan_add_obs(&driverResponseChannel, &sensorSubs, K_MSEC(200));
	zbus_chan_add_obs(&driverResponseChannel, &driverListener, K_NO_WAIT);
}

void MainHandler::Init(){

}

int MainHandler::Deinit(){
    return 0;
}

void MainHandler::MainThread(MainHandler *context){
	// const struct zbus_channel *chan;
	context->currentState = STANDBY;
	// while (!zbus_sub_wait(&stateSubs, &chan, K_MSEC(200))) {
	// 		StateMessage_t state;

	// 		if (&stateChannel == chan) {
	// 				// Indirect message access
	// 				zbus_chan_read(&stateChannel, &state, K_NO_WAIT);
	// 				currentState = state.value[0];
	// 				LOG_DBG("From subscriber -> flag %d data: %d", state.flag, state.value[0]);
	// 		}
	// }
    while (true)
	{
		LOG_DBG("main thread run");
		std::vector<uint8_t> stateVec;
		stateVec.push_back(context->currentState);
		StateMessage_t stateMsg{
			.flag = STATE,
			.value = stateVec
		};
		DriverMessage_t configVector, responseVector;
		zbus_chan_pub(&stateChannel, &stateMsg, K_MSEC(200));

		switch (context->currentState)
		{
		case STANDBY:{
			LOG_INF("Standby");

			configVector.number = LED;
			configVector.type = REQUEST_SET;
			configVector.value.push_back(0);
			context->DriverRequest(&configVector, &responseVector, K_MSEC(200));
			configVector.value.clear();

			configVector.number = SERVO;
			configVector.type = REQUEST_SET;
			configVector.value.push_back(0);
			context->DriverRequest(&configVector, &responseVector, K_MSEC(200));
			configVector.value.clear();

			configVector.number = BLUETOOTH;
			configVector.type = REQUEST_SET;
			configVector.value.push_back(0);
			context->DriverRequest(&configVector, &responseVector, K_MSEC(200));
			configVector.value.clear();

			break;
		}

		case SCANNING:
			LOG_INF("Scanning");
			configVector.number = BLUETOOTH;
			configVector.type = REQUEST_SET;
			configVector.value.push_back(1);
			context->DriverRequest(&configVector, &responseVector, K_MSEC(200));
			configVector.value.clear();
			break;
		
		case USER_1:
			LOG_INF("User 1");
			configVector.number = LED;
			configVector.type = REQUEST_SET;
			configVector.value.push_back(1);
			context->DriverRequest(&configVector, &responseVector, K_MSEC(200));
			configVector.value.clear();

			configVector.number = SERVO;
			configVector.type = REQUEST_SET;
			configVector.value.push_back(0);
			context->DriverRequest(&configVector, &responseVector, K_MSEC(200));
			configVector.value.clear();

			context->currentState = SCANNING;
			break;

		case USER_2:
			LOG_INF("User 2");
			configVector.number = LED;
			configVector.type = REQUEST_SET;
			configVector.value.push_back(0);
			context->DriverRequest(&configVector, &responseVector, K_MSEC(200));
			configVector.value.clear();

			configVector.number = SERVO;
			configVector.type = REQUEST_SET;
			configVector.value.push_back(100);
			context->DriverRequest(&configVector, &responseVector, K_MSEC(200));
			configVector.value.clear();

			context->currentState = SCANNING;
			break;

		default:
			break;
		}
		k_msleep(500);
	}
	
}

void MainHandler::DriverResponseListener(const struct zbus_channel *chan){
	const DriverMessage_t *driver;
	if (&driverResponseChannel == chan) {
			driver = (DriverMessage_t *)zbus_chan_const_msg(chan); // Direct message access
			context->driverResponse_g = *driver;

		if(driver->number == BLUETOOTH){
			if(driver->value[0] == 1){
				context->currentState = USER_1;
			}
			else if(driver->value[0] == 2){
				context->currentState = USER_2;
			}
		}

		if(driver->number == IR_DRIVER){
			if(driver->value.size() > 0){
				if(driver->value[0] == 1){
					if(context->currentState == SCANNING){
						context->currentState = STANDBY;
					}
					else if(context->currentState == STANDBY){
						context->currentState = SCANNING;
					}
				}
			}
		}
	}
}

bool MainHandler::DriverRequest(DriverMessage_t *sentMessage, DriverMessage_t *driverResponse, k_timeout_t timeout){
	zbus_chan_pub(&driverRequestChannel, sentMessage, K_MSEC(200));

	// if(&driverResponse_g == NULL){
	// 	return false;
	// }
	*driverResponse = driverResponse_g;
	// *driverResponse_g = NULL;
    LOG_DBG("data available!");
	return true;
}