#!/usr/bin/env python

import gdb
import rpyc

c = rpyc.connect("localhost", 18080)

def GetState():
    return c.root.getState()

def GetBle():
    return c.root.getBle()

def GetDriver():
    return c.root.getDriver()

def ClearState():
    c.root.clearState()

def RefreshState():
    bpState.enabled = True
    ClearState()

def ClearBle():
    c.root.clearBle()

def ClearDriver():
    c.root.clearDriver()

def ClearAll():
    ClearState()
    ClearBle()
    ClearDriver()

def ClearAndContinue():
    ClearAll()
    bpState.enabled = True
    gdb.execute('continue')

class StateBp(gdb.Breakpoint):
    def __init__(self):
        gdb.Breakpoint.__init__(self, "main-handler.cpp:69")

    def stop(self):
        # SocketStart()
        state = GetState()
        if state >= 0:
            print('set var context->currentState = ' + str(state))
            gdb.execute('set var context->currentState = ' + str(state))
            self.enabled = False
        return False

class DriverBp(gdb.Breakpoint):
    def __init__(self):
        gdb.Breakpoint.__init__(self, "debug-handler.cpp:144")

    def stop(self):
        bleData = GetBle()
        driverData = GetDriver()
        if len(bleData) > 0:
            gdb.execute('set var bleLen = ' + str(len(bleData)))
            gdb.execute('set var isBleAvailable = true')
            # gdb.execute('print bleLen')
            # gdb.execute('print isBleAvailable')
            for i, data in enumerate(bleData):
                gdb.execute('set var bleArray[' +str(i) + '] =' + str(data))
            # gdb.execute('print bleArray')

        if len(driverData) > 0:
            gdb.execute('set var driverLen = ' + str(len(driverData)))
            gdb.execute('set var isDriverAvailable = true')
            for i, data in enumerate(driverData):
                gdb.execute('set var driverArray[' +str(i) + '] =' + str(data))

        return False

bpState = StateBp()
bpDriver = DriverBp()