#!/usr/bin/env python

import socket
import select
import rpyc
from rpyc.utils.server import ThreadedServer

TCP_IP = '192.168.8.149'
TCP_PORT = 12345
BUFFER_SIZE = 1024
MESSAGE = b'ABC'
received = 0

stateData = []
bleData = []
driverData = []

class DebugService(rpyc.Service):
    def on_connect(self,conn):
        print("connected")
        # SocketStart()

    def on_disconnect(self,conn):
        pass

    def parser(self, packet):
        FLAG_BYTE = 0
        LENGTH_MSB = 1
        LENGTH_LSB = 2
        PAYLOAD_BYTE = 3

        STATE_DATA = 0
        BLE_DATA = 1
        DRIVER_DATA = 2

        dataType = FLAG_BYTE
        dataLength = 0
        dataFlag = 0
        
        index = 0
        while index < len(packet):
            dataLength = ((packet[index + 1] << 8) + packet[index + 2])
            if(packet[index] == STATE_DATA):
                stateData.append(packet[index+3])
            elif(packet[index] == BLE_DATA):
                bleData.append(packet[index+3 : index + 3 + dataLength])
            elif(packet[index] == DRIVER_DATA):
                driverData.append(packet[index+3 : index + 3 + dataLength])
            index += (3+dataLength)


    def exposed_socketStart(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((TCP_IP, TCP_PORT))
        s.setblocking(False)
        s.send(MESSAGE)

        inputs = [s]
        outputs = [s]
        while inputs:
            readable,writable,exceptional = select.select(inputs,outputs,inputs,1)
            data = []

            for s in readable:
                data = s.recv(1024)
                self.parser(data)

    def exposed_getState(self):
        if len(stateData) > 0:
            return stateData.pop(0)
        else:
            return -1

    def exposed_getDriver(self):
        if len(driverData) > 0:
            return driverData.pop(0)
        else :
            return []
    
    def exposed_getBle(self):
        if len(bleData) > 0:
            return bleData.pop(0)
        else :
            return []

    def exposed_clearState(self):
        print(bleData)
        stateData.clear()
        print(stateData)

    def exposed_clearDriver(self):
        print(bleData)
        driverData.clear()
        print(driverData)        
    
    def exposed_clearBle(self):
        print(bleData)
        bleData.clear()
        print(bleData)


def SocketStart():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((TCP_IP, TCP_PORT))
    s.setblocking(False)
    s.send(MESSAGE)

    inputs = [s]
    outputs = [s]
    while inputs:
        readable,writable,exceptional = select.select(inputs,outputs,inputs,1)
        data = []

        for s in readable:
            data = s.recv(1024)
            # logging.info(f'Non Blocking - data: {len(data)}')
            # logging.info(f'Non Blocking - closing...')
        if(len(data) >0):
            if(data[0]==0):
                print(data)
                global received
                received = data[3]

ts = ThreadedServer(DebugService,port=18080)
print('Service started on port 18080')
ts.start()